import { FSA } from "./types";
export declare function setDefaultTimeout(timeout: number): void;
export declare function getDefaultTimeout(): number;
export declare function setDefaultTimeoutHandler(func: (optimisticId: string, commit: Function, rollback: Function) => void): void;
export declare function getDefaultTimeoutHandler(): (optimisticId: string, commit: Function, rollback: Function) => void;
/**
 * Simplifies the usage of redux-optimism. By calling this function with the dispatch function of your store,
 * redux-optimism will dispatch an 'INIT_OPTIMISM' action with a newly created ID and give you back three functions:
 *  * makeOptimistic
 *  * commit
 *  * rollback
 * With makeOptimistic(action), you can optimistically dispatch an action. With commit(), the previously executed makeOptimistic
 * actions will be manifested. And with rollback(), the state will be the one, that would have occurred without the
 * optimistically dispatched actions.
 * @param dispatch
 * @returns {{makeOptimistic: (action: FSA) => never, rollback: () => never, timer: number, dispatched: never, commit: () => never}}
 * @param timeout
 * @param handleTimeout
 */
export declare function getOptimism(dispatch: Function, timeout?: number, handleTimeout?: (optimisticId: string, commit: Function, rollback: Function) => void): {
    optimistic: (action: FSA) => any;
    commit: () => any;
    rollback: () => any;
    dispatched: any;
    timer: number;
};
/**
 * By dispatching this action, the actions, that were dispatched before all active optimisms, will be integrated into a
 * new realistic state. Saves time, when there is an rollback. Should be dispatched in idle time.
 * @type {{type: optimisticActions.cleanup}}
 */
export declare const cleanStore: FSA;
