import { Reducer } from "./types";
/**
 * The identity operator.
 * @param x
 * @returns {any}
 */
export declare function identity(x: any): typeof x;
/**
 * Map function for objects. The first argument is the object, the second argument is a function that takes a tuple
 * [key, value] and returns a tuple [newKey, newValue]
 * @param object
 * @param mapFunction
 * @returns {any}
 */
export declare function objectMap(object: any, mapFunction?: ([key, value]: [string, any]) => [string, any]): typeof object;
/**
 * Filter function for objects. The first argument is the object, the filterFunction takes a tuple [key, value] and
 * returns a boolean.
 * @param object
 * @param filterFunction
 * @returns {any}
 */
export declare function objectFilter(object: any, filterFunction?: ([key, value]: [string, any]) => boolean): typeof object;
/**
 * Calculates the intersection between arrays.
 * @param arrays
 * @returns {Array<any>}
 */
export declare function intersect(...arrays: Array<any>): Array<any>;
/**
 * Enhances a reducer with optimism.
 * @param wrappedReducer
 * @param actionIdGenerator
 */
export declare function makeOptimistic(wrappedReducer: Reducer, actionIdGenerator?: (...args: Array<any>) => string): Reducer;
/**
 * Cleanup the state.
 * @param previousState
 * @param action
 * @param actionId
 * @param wrappedReducer
 * @returns {any}
 */
/**
function cleanup(
 previousState: optimisticState,
 action: FSA,
 actionId: string,
 wrappedReducer: Reducer): optimisticState {

    if (Object.keys(previousState.optimism.optimisms).length === 0) {
        return {
            ...previousState,
            optimism: {
                ...previousState.optimism,
                actions: {},
                actionsUntilNow: [],
                realisticState: objectFilter(
                    previousState,
                    ([key, value]: [string, any]) => key != 'optimism'
                )
            }
        }

    } else {
        const commonActions = intersect(
            ...Object.values(previousState.optimism.optimisms).map(
                (optimism: any) => optimism.previousActions)
        )

        const newRealisticState = commonActions.reduce(
            (accumulatedState: any, action: FSA) => {
                return makeOptimistic(wrappedReducer)(accumulatedState, action)
            },
            previousState.optimism.realisticState)

        return {
            ...previousState,
            optimism: {

                ...previousState.optimism,

                actions: objectFilter(
                    previousState.optimism.actions,
                    ([actionId, action]: [string, FSA]) => !commonActions.includes(actionId)
                ),

                actionsUntilNow: previousState.optimism.actionsUntilNow.filter(
                    (actionId: string) => !commonActions.includes(actionId)
                ),

                optimisms: objectMap(
                    previousState.optimism.optimisms,
                    ([optimismId, optimism]: [string, optimism]) => [optimismId, {
                        ...optimism,
                        previousActions: optimism.previousActions.filter(
                            (actionId: string) => !commonActions.includes(actionId)
                        )
                    }]),

                realisticState: objectFilter(
                    newRealisticState,
                    ([key, value]: [string, any]) => key != 'optimism'
                )
            }
        }
    }
}
 */
